# sb::Data::Category
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Data::Category;
use strict;
use sb::Config ();
use sb::Data::Object ();
use base qw( sb::Data::Object );
use vars qw( $VERSION );
$VERSION = '0.12';
# 0.12 [2008/07/18] modified 'disp' in the table
# 0.11 [2007/08/14] added deprecated functions
# 0.10 [2007/08/07] changed fullname to change "main"
# 0.09 [2007/08/04] added initializer
# 0.08 [2007/07/16] changed return to make them consistent
# 0.07 [2007/02/09] modified add_sub / remove_sub to handle sub-categories correctly
# 0.06 [2007/02/07] added formatted_text
# 0.05 [2007/02/06] changed %aOptionKeys to add line/sum options
# 0.04 [2006/07/01] ported from biscuit
# 0.03 [2005/07/22] changed data structure to array
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/06/01] added 'dir' method
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines / static variables
sub CATEGORY_FORMAT (){ '%Main% &gt; %Sub%' }
sub DEFAULT_OPTION  (){ '0:0:0:1:' };
my %aOptionKeys = (
	'top'  => 0,  # display setting for top page
	'list' => 1,  # display setting for list
	'line' => 2,  # linefeed setting for description
	'sum'  => 3,  # display setting for summary on the list
);
# --------------------------------------
# // override base class - table
sub table
{
	my $self = shift;
	return (
		'id'    => 'uni_id',   # id
		'wid'   => 'id',       # weblog id (reserved)
		'name'  => 'str(255)', # category name
		'text'  => 'text',     # description
		'url'   => 'str(255)', # default trackback url to send ping
		'main'  => 'id',       # parent category
		'order' => 'int',      # order of list
		'temp'  => 'id',       # template id
		'dir'   => 'str(255)', # stored directory
		'disp'  => 'str(255)', # display setting
		'sub'   => 'str(255)', # sub categories
		'num'   => 'int',      # number of articles
		'idx'   => 'int',      # category index file
	);
}
# --------------------------------------
# // initializer
BEGIN
{
	__PACKAGE__->init_accessor();
}
sub default
{ # default settings
	my $self  = shift;
	my %param = @_;
	$param{'wid'} |= 0;
	$param{'main'} = -1 if ($param{'main'} eq '');
	$param{'order'} |= $self->id;
	$param{'temp'} = -1 if ($param{'temp'} eq '');
	$param{'disp'} = DEFAULT_OPTION if ($param{'disp'} eq '');
	$param{'num'} |= 0;
	$param{'idx'} |= 0;
	$self->SUPER::initialize(%param);
}
# --------------------------------------
# // public functions
sub dir
{
	my $self = shift;
	$self->{'dir'} = shift if @_;
	return ($self->{'dir'} ne '')
		? $self->{'dir'}
		: sb::Config->get->value('conf_dir_log');
}
sub get_option
{
	my $self = shift;
	my $key  = shift;
	return if (!$key);
	return if (! defined($aOptionKeys{$key}));
	my @option = split(':',$self->disp);
	return $option[$aOptionKeys{$key}];
}
sub set_option
{
	my $self  = shift;
	my $key   = shift;
	my $value = shift;
	return if (!$key);
	return if (! defined($aOptionKeys{$key}));
	my @option = split(':',$self->disp);
	$option[$aOptionKeys{$key}] = int($value);
	$self->disp(join(':',@option) . ':');
}
sub get_option_keys
{
	my $self = shift;
	return keys(%aOptionKeys);
}
sub add_sub
{
	my $self = shift;
	my $add  = shift;
	my $sub  = ($add ne '') ? $add . ',' . $self->sub : $self->sub;
	$sub =~ s/,,/,/g;
	$sub = '' if ($sub eq ',');
	return $self->sub($sub);
}
sub remove_sub
{
	my $self = shift;
	my $del  = shift;
	my @subs = ();
	foreach ( split(',',$self->sub) )
	{
		push(@subs,$_) if ($_ ne $del);
	}
	my $sub = '';
	$sub = join(',',@subs) . ',' if (@subs);
	return $self->sub($sub);
}
sub fullname_with_link
{
	my $self = shift;
	return '<a href="' . $self->cat_url . '">' . $self->fullname(@_) . '</a>';
}
sub cat_url
{
	my $self = shift;
	return ($self->idx)
		? sb::Config->get->value('conf_srv_base') . $self->dir
		: sb::Config->get->value('conf_srv_cgi') . sb::Config->get->value('basic_sb') . '?cid=' . $self->id;
}
sub fullname
{
	my $self = shift;
	my $cat  = shift;
	my $form = (@_) ? shift : CATEGORY_FORMAT;
	return $self->name if (! defined($cat));
	return $self->name if ($self->main eq '' or $self->main == -1);
	return $self->name if (! defined($cat->{$self->main}));
	my $main = $cat->{$self->main}->fullname($cat,$form);
	my $name = $self->name;
	$form =~ s/%Main%/$main/;
	$form =~ s/%Sub%/$name/;
	return $form;
}
sub formatted_text
{
	my $self = shift;
	my $as_summary = shift;
	return if ($self->text eq '');
	my $text = $self->text;
	$text = sb::Text->new($text)->format('autobreak') if ($self->get_option('line'));
	$text = sb::Text->new($text)->clip if ($as_summary);
	return $text;
}
# --------------------------------------
# // public functions - deprecated
sub formated_text
{
	formatted_text(@_);
}
# --------------------------------------
# // private functions
1;
__END__
