# sb::Template
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Template;
use strict;
use integer;
use sb::Utils ();
use vars qw( $VERSION );
$VERSION = '0.10';
# 0.10 [2007/11/28] modified to use sb::Utils->remove_duplicates
# 0.09 [2007/08/04] got rid of unnecessary dependencies
# 0.08 [2007/07/19] added unifyFor for backward compatibility
# 0.07 [2007/07/16] changed return to make them consistent
# 0.06 [2007/07/06] refactoring / renamed
# 0.05 [2006/07/04] got rid of "return( undef )"
# 0.04 [2006/03/17] changed bracket rule
# 0.03 [2005/11/15] porting from Serene Bach
# 0.02 [2005/10/19] changed block structure to store contents as an array instead of text
# 0.01 [2005/10/18] added clear
# 0.00 [2004/11/17] generated
# --------------------------------------
# // defines / static variables
sub BLOCKS (){ 0 }
sub DATA   (){ 1 }
sub NUMBER (){ 2 }
sub TAG    (){ 0 }
sub BLOCK  (){ 1 }
sub TEXT   (){ 0 }
sub TAGS   (){ 1 }
# --------------------------------------
# // override base class
sub new
{
	my $class = shift;
	my $template = shift;
	die('Need to set template') if ( !$template );
	my $self = bless [], $class;
	$self->[BLOCKS] = _parse($template); # block structure
	$self->[NUMBER] = 0;  # number of proceeded block
	$self->[DATA]   = []; # hash for data
	$self->[DATA][TAG]   = {};
	$self->[DATA][BLOCK] = {};
	return $self;
}
# --------------------------------------
# // public functions
sub output
{ # evolving template
	my $self = shift;
	$self->[DATA][BLOCK]{'-main'} = 1;
	return $self->_output_block('-main');
}
sub deleteBlock
{ # delete block
	my $self = shift;
	my $block = shift; # block name to delete
	die('Lack of parameter') if(! defined($block));
	if ( exists($self->[BLOCKS]{$block}[TEXT]) )
	{
		$self->[BLOCKS]{$block}[TEXT] = '';
	}
	return;
}
sub clear
{
	my $self = shift;
	$self->[DATA][TAG] = {};
	foreach my $block ( keys(%{$self->[DATA][BLOCK]}) )
	{
		$self->[DATA][BLOCK]{$block} = 0;
	}
}
sub tag
{
	my $self = shift;
	my ($tag,$content) = @_; # tag name, content
	my $num = $self->[NUMBER];
	die('Lack of parameters') if (! defined($tag));
	$self->[DATA][TAG]{'{' . $tag . '}'}[$num] = $content;
}
sub num
{
	my $self = shift;
	$self->[NUMBER] = shift;
}
sub block
{
	my $self  = shift;
	my ($block,$num) = @_; # block name, repetition number
	die('Lack of parameters') if (! defined($block) or ! defined($num));
	$self->[DATA][BLOCK]{$block} = $num;
}
sub existed
{
	my $self  = shift;
	my $block = shift;
	return if (! defined($block));
	return exists($self->[BLOCKS]{$block});
}
sub unifyFor
{
	my $self = shift;
	return;
}
# --------------------------------------
# // private functions
sub _output_block
{
	my $self = shift;
	my $name = shift;
	my $output = '';
	return if (! defined($name));
	return if (!$self->existed($name));
	return if (!$self->[DATA][BLOCK]{$name});
	return if ($self->[BLOCKS]{$name}[TEXT] eq '');
	for my $i (0 .. $self->[DATA][BLOCK]{$name} - 1)
	{
		my $text = $self->[BLOCKS]{$name}[TEXT];
		foreach my $tag (@{$self->[BLOCKS]{$name}[TAGS]})
		{
			my $result = 
				  ($tag =~ /\{\-(\w+)\}/)               ? $self->_output_block($1)
				: defined($self->[DATA][TAG]{$tag}[$i]) ? $self->[DATA][TAG]{$tag}[$i]
				: defined($self->[DATA][TAG]{$tag}[0])  ? $self->[DATA][TAG]{$tag}[0]
				:                                         '';
			$text =~ s/\Q$tag\E/$result/g;
		}
		$output .= $text;
	}
	return $output;
}
sub _parse
{
	my $template = shift;
	my %blocks = ();
	my @stack = ();
	my $cur = '-main';
	$blocks{$cur} = ['',[]];
	push(@stack,$cur);
	foreach my $line (split("\n",$template))
	{
		if ($line =~ /<\!-- BEGIN (\w+) -->/)
		{
			$blocks{$cur}[TEXT] .= '{-' . $1 . '}';
			push(@{$blocks{$cur}[TAGS]},'{-' . $1 . '}');
			$cur = $1;
			$blocks{$cur} = ['',[]];
			push(@stack,$cur);
			next;
		}
		elsif (index($line,'<!-- END ') > -1)
		{
			@{$blocks{$cur}[TAGS]} = sb::Utils->remove_duplicates(@{$blocks{$cur}[TAGS]});
			pop(@stack);
			$cur = $stack[$#stack];
			next;
		}
		if ($line =~ /\{\w+?\}/)
		{
			push(@{$blocks{$cur}[TAGS]},($line =~ /(\{\w+?\})/g));
		}
		$blocks{$cur}[TEXT] .= $line . "\n";
	}
	return \%blocks;
}
1;
__END__
